@@ -17,6 +17,7 @@ from homeassistant.const import (
     TEMP_CELSIUS,
 )
 from homeassistant.core import HomeAssistant
+from homeassistant.helpers.entity import DeviceInfo
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.update_coordinator import CoordinatorEntity
 
@@ -114,13 +115,13 @@ class CanarySensor(CoordinatorEntity, SensorEntity):
 
         self._canary_type = canary_sensor_type
         self._attr_unique_id = f"{device.device_id}_{sensor_type[0]}"
-        self._attr_device_info = {
-            "identifiers": {(DOMAIN, str(device.device_id))},
-            "name": device.name,
-            "model": device.device_type["name"],
-            "manufacturer": MANUFACTURER,
-        }
-        self._attr_unit_of_measurement = sensor_type[1]
+        self._attr_device_info = DeviceInfo(
+            identifiers={(DOMAIN, str(device.device_id))},
+            model=device.device_type["name"],
+            manufacturer=MANUFACTURER,
+            name=device.name,
+        )
+        self._attr_native_unit_of_measurement = sensor_type[1]
         self._attr_device_class = sensor_type[3]
         self._attr_icon = sensor_type[2]
 
@@ -144,7 +145,7 @@ class CanarySensor(CoordinatorEntity, SensorEntity):
         return None
 
     @property
-    def state(self) -> float | None:
+    def native_value(self) -> float | None:
         """Return the state of the sensor."""
         return self.reading
 
