@@ -1,24 +1,18 @@
 """Config flow for DoorBird integration."""
+from http import HTTPStatus
 from ipaddress import ip_address
 import logging
-import urllib
 
 from doorbirdpy import DoorBird
+import requests
 import voluptuous as vol
 
 from homeassistant import config_entries, core, exceptions
-from homeassistant.const import (
-    CONF_HOST,
-    CONF_NAME,
-    CONF_PASSWORD,
-    CONF_USERNAME,
-    HTTP_UNAUTHORIZED,
-)
+from homeassistant.const import CONF_HOST, CONF_NAME, CONF_PASSWORD, CONF_USERNAME
 from homeassistant.core import callback
 from homeassistant.util.network import is_link_local
 
-from .const import CONF_EVENTS, DOORBIRD_OUI
-from .const import DOMAIN  # pylint:disable=unused-import
+from .const import CONF_EVENTS, DOMAIN, DOORBIRD_OUI
 from .util import get_mac_address_from_doorstation_info
 
 _LOGGER = logging.getLogger(__name__)
@@ -35,17 +29,18 @@ def _schema_with_defaults(host=None, name=None):
     )
 
 
-async def validate_input(hass: core.HomeAssistant, data):
-    """Validate the user input allows us to connect.
+def _check_device(device):
+    """Verify we can connect to the device and return the status."""
+    return device.ready(), device.info()
 
-    Data has the keys from DATA_SCHEMA with values provided by the user.
-    """
+
+async def validate_input(hass: core.HomeAssistant, data):
+    """Validate the user input allows us to connect."""
     device = DoorBird(data[CONF_HOST], data[CONF_USERNAME], data[CONF_PASSWORD])
     try:
-        status = await hass.async_add_executor_job(device.ready)
-        info = await hass.async_add_executor_job(device.info)
-    except urllib.error.HTTPError as err:
-        if err.code == HTTP_UNAUTHORIZED:
+        status, info = await hass.async_add_executor_job(_check_device, device)
+    except requests.exceptions.HTTPError as err:
+        if err.response.status_code == HTTPStatus.UNAUTHORIZED:
             raise InvalidAuth from err
         raise CannotConnect from err
     except OSError as err:
@@ -60,11 +55,23 @@ async def validate_input(hass: core.HomeAssistant, data):
     return {"title": data[CONF_HOST], "mac_addr": mac_addr}
 
 
+async def async_verify_supported_device(hass, host):
+    """Verify the doorbell state endpoint returns a 401."""
+    device = DoorBird(host, "", "")
+    try:
+        await hass.async_add_executor_job(device.doorbell_state)
+    except requests.exceptions.HTTPError as err:
+        if err.response.status_code == HTTPStatus.UNAUTHORIZED:
+            return True
+    except OSError:
+        return False
+    return False
+
+
 class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
     """Handle a config flow for DoorBird."""
 
     VERSION = 1
-    CONNECTION_CLASS = config_entries.CONN_CLASS_LOCAL_PUSH
 
     def __init__(self):
         """Initialize the DoorBird config flow."""
@@ -86,17 +93,20 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
     async def async_step_zeroconf(self, discovery_info):
         """Prepare configuration for a discovered doorbird device."""
         macaddress = discovery_info["properties"]["macaddress"]
+        host = discovery_info[CONF_HOST]
 
         if macaddress[:6] != DOORBIRD_OUI:
             return self.async_abort(reason="not_doorbird_device")
-        if is_link_local(ip_address(discovery_info[CONF_HOST])):
+        if is_link_local(ip_address(host)):
             return self.async_abort(reason="link_local_address")
 
         await self.async_set_unique_id(macaddress)
+        self._abort_if_unique_id_configured(updates={CONF_HOST: host})
 
-        self._abort_if_unique_id_configured(
-            updates={CONF_HOST: discovery_info[CONF_HOST]}
-        )
+        self._async_abort_entries_match({CONF_HOST: host})
+
+        if not await async_verify_supported_device(self.hass, host):
+            return self.async_abort(reason="not_doorbird_device")
 
         chop_ending = "._axis-video._tcp.local."
         friendly_hostname = discovery_info["name"]
@@ -105,26 +115,12 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
 
         self.context["title_placeholders"] = {
             CONF_NAME: friendly_hostname,
-            CONF_HOST: discovery_info[CONF_HOST],
+            CONF_HOST: host,
         }
-        self.discovery_schema = _schema_with_defaults(
-            host=discovery_info[CONF_HOST], name=friendly_hostname
-        )
+        self.discovery_schema = _schema_with_defaults(host=host, name=friendly_hostname)
 
         return await self.async_step_user()
 
-    async def async_step_import(self, user_input):
-        """Handle import."""
-        if user_input:
-            info, errors = await self._async_validate_or_error(user_input)
-            if not errors:
-                await self.async_set_unique_id(
-                    info["mac_addr"], raise_on_progress=False
-                )
-                self._abort_if_unique_id_configured()
-                return self.async_create_entry(title=info["title"], data=user_input)
-        return await self.async_step_user(user_input)
-
     async def _async_validate_or_error(self, user_input):
         """Validate doorbird or error."""
         errors = {}
@@ -150,7 +146,7 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
 class OptionsFlowHandler(config_entries.OptionsFlow):
     """Handle a option flow for doorbird."""
 
-    def __init__(self, config_entry: config_entries.ConfigEntry):
+    def __init__(self, config_entry: config_entries.ConfigEntry) -> None:
         """Initialize options flow."""
         self.config_entry = config_entry
 
