@@ -14,17 +14,19 @@ from homeassistant.const import (
     RESTART_EXIT_CODE,
     SERVICE_HOMEASSISTANT_RESTART,
     SERVICE_HOMEASSISTANT_STOP,
+    SERVICE_SAVE_PERSISTENT_STATES,
     SERVICE_TOGGLE,
     SERVICE_TURN_OFF,
     SERVICE_TURN_ON,
 )
 import homeassistant.core as ha
 from homeassistant.exceptions import HomeAssistantError, Unauthorized, UnknownUser
-from homeassistant.helpers import config_validation as cv, recorder
+from homeassistant.helpers import config_validation as cv, recorder, restore_state
 from homeassistant.helpers.service import (
     async_extract_config_entry_ids,
     async_extract_referenced_entity_ids,
 )
+from homeassistant.helpers.typing import ConfigType
 
 ATTR_ENTRY_ID = "entry_id"
 
@@ -50,12 +52,16 @@ SCHEMA_RELOAD_CONFIG_ENTRY = vol.All(
 SHUTDOWN_SERVICES = (SERVICE_HOMEASSISTANT_STOP, SERVICE_HOMEASSISTANT_RESTART)
 
 
-async def async_setup(hass: ha.HomeAssistant, config: dict) -> bool:  # noqa: C901
+async def async_setup(hass: ha.HomeAssistant, config: ConfigType) -> bool:  # noqa: C901
     """Set up general services related to Home Assistant."""
 
+    async def async_save_persistent_states(service):
+        """Handle calls to homeassistant.save_persistent_states."""
+        await restore_state.RestoreStateData.async_save_persistent_states(hass)
+
     async def async_handle_turn_service(service):
         """Handle calls to homeassistant.turn_on/off."""
-        referenced = await async_extract_referenced_entity_ids(hass, service)
+        referenced = async_extract_referenced_entity_ids(hass, service)
         all_referenced = referenced.referenced | referenced.indirectly_referenced
 
         # Generic turn on/off method requires entity id
@@ -114,6 +120,10 @@ async def async_setup(hass: ha.HomeAssistant, config: dict) -> bool:  # noqa: C9
         if tasks:
             await asyncio.gather(*tasks)
 
+    hass.services.async_register(
+        ha.DOMAIN, SERVICE_SAVE_PERSISTENT_STATES, async_save_persistent_states
+    )
+
     service_schema = vol.Schema({ATTR_ENTITY_ID: cv.entity_ids}, extra=vol.ALLOW_EXTRA)
 
     hass.services.async_register(
@@ -133,11 +143,12 @@ async def async_setup(hass: ha.HomeAssistant, config: dict) -> bool:  # noqa: C9
             and await recorder.async_migration_in_progress(hass)
         ):
             _LOGGER.error(
-                "The system cannot %s while a database upgrade in progress",
+                "The system cannot %s while a database upgrade is in progress",
                 call.service,
             )
             raise HomeAssistantError(
-                f"The system cannot {call.service} while a database upgrade in progress."
+                f"The system cannot {call.service} "
+                "while a database upgrade is in progress."
             )
 
         if call.service == SERVICE_HOMEASSISTANT_STOP:
@@ -158,7 +169,8 @@ async def async_setup(hass: ha.HomeAssistant, config: dict) -> bool:  # noqa: C9
                 f"{ha.DOMAIN}.check_config",
             )
             raise HomeAssistantError(
-                f"The system cannot {call.service} because the configuration is not valid: {errors}"
+                f"The system cannot {call.service} "
+                f"because the configuration is not valid: {errors}"
             )
 
         if call.service == SERVICE_HOMEASSISTANT_RESTART:
@@ -246,10 +258,10 @@ async def async_setup(hass: ha.HomeAssistant, config: dict) -> bool:  # noqa: C9
         if not reload_entries:
             raise ValueError("There were no matching config entries to reload")
         await asyncio.gather(
-            *[
+            *(
                 hass.config_entries.async_reload(config_entry_id)
                 for config_entry_id in reload_entries
-            ]
+            )
         )
 
     hass.helpers.service.async_register_admin_service(
