@@ -1,11 +1,17 @@
 """Ban logic for HTTP component."""
+from __future__ import annotations
+
 from collections import defaultdict
+from collections.abc import Awaitable, Callable
+from contextlib import suppress
 from datetime import datetime
+from http import HTTPStatus
 from ipaddress import ip_address
 import logging
-from typing import List, Optional
+from socket import gethostbyaddr, herror
+from typing import Any, Final
 
-from aiohttp.web import middleware
+from aiohttp.web import Application, Request, StreamResponse, middleware
 from aiohttp.web_exceptions import HTTPForbidden, HTTPUnauthorized
 import voluptuous as vol
 
@@ -13,37 +19,35 @@ from homeassistant.config import load_yaml_config_file
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.exceptions import HomeAssistantError
 import homeassistant.helpers.config_validation as cv
-from homeassistant.util.yaml import dump
+from homeassistant.util import dt as dt_util, yaml
 
-from .const import KEY_REAL_IP
+from .view import HomeAssistantView
 
-# mypy: allow-untyped-defs, no-check-untyped-defs
+_LOGGER: Final = logging.getLogger(__name__)
 
-_LOGGER = logging.getLogger(__name__)
+KEY_BANNED_IPS: Final = "ha_banned_ips"
+KEY_FAILED_LOGIN_ATTEMPTS: Final = "ha_failed_login_attempts"
+KEY_LOGIN_THRESHOLD: Final = "ha_login_threshold"
 
-KEY_BANNED_IPS = "ha_banned_ips"
-KEY_FAILED_LOGIN_ATTEMPTS = "ha_failed_login_attempts"
-KEY_LOGIN_THRESHOLD = "ha_login_threshold"
+NOTIFICATION_ID_BAN: Final = "ip-ban"
+NOTIFICATION_ID_LOGIN: Final = "http-login"
 
-NOTIFICATION_ID_BAN = "ip-ban"
-NOTIFICATION_ID_LOGIN = "http-login"
+IP_BANS_FILE: Final = "ip_bans.yaml"
+ATTR_BANNED_AT: Final = "banned_at"
 
-IP_BANS_FILE = "ip_bans.yaml"
-ATTR_BANNED_AT = "banned_at"
-
-SCHEMA_IP_BAN_ENTRY = vol.Schema(
+SCHEMA_IP_BAN_ENTRY: Final = vol.Schema(
     {vol.Optional("banned_at"): vol.Any(None, cv.datetime)}
 )
 
 
 @callback
-def setup_bans(hass, app, login_threshold):
+def setup_bans(hass: HomeAssistant, app: Application, login_threshold: int) -> None:
     """Create IP Ban middleware for the app."""
     app.middlewares.append(ban_middleware)
     app[KEY_FAILED_LOGIN_ATTEMPTS] = defaultdict(int)
     app[KEY_LOGIN_THRESHOLD] = login_threshold
 
-    async def ban_startup(app):
+    async def ban_startup(app: Application) -> None:
         """Initialize bans when app starts up."""
         app[KEY_BANNED_IPS] = await async_load_ip_bans_config(
             hass, hass.config.path(IP_BANS_FILE)
@@ -53,14 +57,16 @@ def setup_bans(hass, app, login_threshold):
 
 
 @middleware
-async def ban_middleware(request, handler):
+async def ban_middleware(
+    request: Request, handler: Callable[[Request], Awaitable[StreamResponse]]
+) -> StreamResponse:
     """IP Ban middleware."""
     if KEY_BANNED_IPS not in request.app:
         _LOGGER.error("IP Ban middleware loaded but banned IPs not loaded")
         return await handler(request)
 
     # Verify if IP is not banned
-    ip_address_ = request[KEY_REAL_IP]
+    ip_address_ = ip_address(request.remote)
     is_banned = any(
         ip_ban.ip_address == ip_address_ for ip_ban in request.app[KEY_BANNED_IPS]
     )
@@ -75,33 +81,50 @@ async def ban_middleware(request, handler):
         raise
 
 
-def log_invalid_auth(func):
+def log_invalid_auth(
+    func: Callable[..., Awaitable[StreamResponse]]
+) -> Callable[..., Awaitable[StreamResponse]]:
     """Decorate function to handle invalid auth or failed login attempts."""
 
-    async def handle_req(view, request, *args, **kwargs):
-        """Try to log failed login attempts if response status >= 400."""
+    async def handle_req(
+        view: HomeAssistantView, request: Request, *args: Any, **kwargs: Any
+    ) -> StreamResponse:
+        """Try to log failed login attempts if response status >= BAD_REQUEST."""
         resp = await func(view, request, *args, **kwargs)
-        if resp.status >= 400:
+        if resp.status >= HTTPStatus.BAD_REQUEST:
             await process_wrong_login(request)
         return resp
 
     return handle_req
 
 
-async def process_wrong_login(request):
+async def process_wrong_login(request: Request) -> None:
     """Process a wrong login attempt.
 
     Increase failed login attempts counter for remote IP address.
     Add ip ban entry if failed login attempts exceeds threshold.
     """
-    remote_addr = request[KEY_REAL_IP]
-
-    msg = f"Login attempt or request with invalid authentication from {remote_addr}"
-    _LOGGER.warning(msg)
-
     hass = request.app["hass"]
+
+    remote_addr = ip_address(request.remote)
+    remote_host = request.remote
+    with suppress(herror):
+        remote_host, _, _ = await hass.async_add_executor_job(
+            gethostbyaddr, request.remote
+        )
+
+    base_msg = f"Login attempt or request with invalid authentication from {remote_host} ({remote_addr})."
+
+    # The user-agent is unsanitized input so we only include it in the log
+    user_agent = request.headers.get("user-agent")
+    log_msg = f"{base_msg} ({user_agent})"
+
+    notification_msg = f"{base_msg} See the log for details."
+
+    _LOGGER.warning(log_msg)
+
     hass.components.persistent_notification.async_create(
-        msg, "Login attempt failed", NOTIFICATION_ID_LOGIN
+        notification_msg, "Login attempt failed", NOTIFICATION_ID_LOGIN
     )
 
     # Check if ban middleware is loaded
@@ -110,6 +133,13 @@ async def process_wrong_login(request):
 
     request.app[KEY_FAILED_LOGIN_ATTEMPTS][remote_addr] += 1
 
+    # Supervisor IP should never be banned
+    if (
+        "hassio" in hass.config.components
+        and hass.components.hassio.get_supervisor_ip() == str(remote_addr)
+    ):
+        return
+
     if (
         request.app[KEY_FAILED_LOGIN_ATTEMPTS][remote_addr]
         >= request.app[KEY_LOGIN_THRESHOLD]
@@ -117,7 +147,7 @@ async def process_wrong_login(request):
         new_ban = IpBan(remote_addr)
         request.app[KEY_BANNED_IPS].append(new_ban)
 
-        await hass.async_add_job(
+        await hass.async_add_executor_job(
             update_ip_bans_config, hass.config.path(IP_BANS_FILE), new_ban
         )
 
@@ -130,14 +160,14 @@ async def process_wrong_login(request):
         )
 
 
-async def process_success_login(request):
+async def process_success_login(request: Request) -> None:
     """Process a success login attempt.
 
     Reset failed login attempts counter for remote IP address.
     No release IP address from banned list function, it can only be done by
     manual modify ip bans config file.
     """
-    remote_addr = request[KEY_REAL_IP]
+    remote_addr = ip_address(request.remote)
 
     # Check if ban middleware is loaded
     if KEY_BANNED_IPS not in request.app or request.app[KEY_LOGIN_THRESHOLD] < 1:
@@ -156,15 +186,15 @@ async def process_success_login(request):
 class IpBan:
     """Represents banned IP address."""
 
-    def __init__(self, ip_ban: str, banned_at: Optional[datetime] = None) -> None:
+    def __init__(self, ip_ban: str, banned_at: datetime | None = None) -> None:
         """Initialize IP Ban object."""
         self.ip_address = ip_address(ip_ban)
-        self.banned_at = banned_at or datetime.utcnow()
+        self.banned_at = banned_at or dt_util.utcnow()
 
 
-async def async_load_ip_bans_config(hass: HomeAssistant, path: str) -> List[IpBan]:
+async def async_load_ip_bans_config(hass: HomeAssistant, path: str) -> list[IpBan]:
     """Load list of banned IPs from config file."""
-    ip_list: List[IpBan] = []
+    ip_list: list[IpBan] = []
 
     try:
         list_ = await hass.async_add_executor_job(load_yaml_config_file, path)
@@ -187,11 +217,7 @@ async def async_load_ip_bans_config(hass: HomeAssistant, path: str) -> List[IpBa
 
 def update_ip_bans_config(path: str, ip_ban: IpBan) -> None:
     """Update config file with new banned IP address."""
-    with open(path, "a") as out:
-        ip_ = {
-            str(ip_ban.ip_address): {
-                ATTR_BANNED_AT: ip_ban.banned_at.strftime("%Y-%m-%dT%H:%M:%S")
-            }
-        }
+    with open(path, "a", encoding="utf8") as out:
+        ip_ = {str(ip_ban.ip_address): {ATTR_BANNED_AT: ip_ban.banned_at.isoformat()}}
         out.write("\n")
-        out.write(dump(ip_))
+        out.write(yaml.dump(ip_))
