@@ -1,6 +1,7 @@
 """Schemas used by insteon component."""
+from __future__ import annotations
+
 from binascii import Error as HexError, unhexlify
-from typing import Dict
 
 from pyinsteon.address import Address
 from pyinsteon.constants import HC_LOOKUP
@@ -39,6 +40,7 @@ from .const import (
     CONF_X10_ALL_UNITS_OFF,
     DOMAIN,
     HOUSECODES,
+    INSTEON_ADDR_REGEX,
     PORT_HUB_V1,
     PORT_HUB_V2,
     SRV_ALL_LINK_GROUP,
@@ -51,18 +53,24 @@ from .const import (
 )
 
 
-def set_default_port(schema: Dict) -> Dict:
+def set_default_port(schema: dict) -> dict:
     """Set the default port based on the Hub version."""
     # If the ip_port is found do nothing
     # If it is not found the set the default
-    ip_port = schema.get(CONF_IP_PORT)
-    if not ip_port:
+    if not schema.get(CONF_IP_PORT):
         hub_version = schema.get(CONF_HUB_VERSION)
         # Found hub_version but not ip_port
         schema[CONF_IP_PORT] = PORT_HUB_V1 if hub_version == 1 else PORT_HUB_V2
     return schema
 
 
+def insteon_address(value: str) -> str:
+    """Validate an Insteon address."""
+    if not INSTEON_ADDR_REGEX.match(value):
+        raise vol.Invalid("Invalid Insteon Address")
+    return str(value).replace(".", "").lower()
+
+
 CONF_DEVICE_OVERRIDE_SCHEMA = vol.All(
     vol.Schema(
         {
@@ -160,7 +168,7 @@ TRIGGER_SCENE_SCHEMA = vol.Schema(
 ADD_DEFAULT_LINKS_SCHEMA = vol.Schema({vol.Required(CONF_ENTITY_ID): cv.entity_id})
 
 
-def normalize_byte_entry_to_int(entry: [int, bytes, str]):
+def normalize_byte_entry_to_int(entry: int | bytes | str):
     """Format a hex entry value."""
     if isinstance(entry, int):
         if entry in range(0, 256):
