@@ -3,7 +3,7 @@ from __future__ import annotations
 
 from abc import ABC
 from dataclasses import dataclass
-from typing import List, Optional, Tuple
+from typing import Any, cast
 
 from homeassistant.components.media_player import BrowseMedia
 from homeassistant.components.media_player.const import (
@@ -28,9 +28,11 @@ class PlayMedia:
 class BrowseMediaSource(BrowseMedia):
     """Represent a browsable media file."""
 
-    children: Optional[List["BrowseMediaSource"]]
+    children: list[BrowseMediaSource | BrowseMedia] | None
 
-    def __init__(self, *, domain: Optional[str], identifier: Optional[str], **kwargs):
+    def __init__(
+        self, *, domain: str | None, identifier: str | None, **kwargs: Any
+    ) -> None:
         """Initialize media source browse media."""
         media_content_id = f"{URI_SCHEME}{domain or ''}"
         if identifier:
@@ -47,7 +49,7 @@ class MediaSourceItem:
     """A parsed media item."""
 
     hass: HomeAssistant
-    domain: Optional[str]
+    domain: str | None
     identifier: str
 
     async def async_browse(self) -> BrowseMediaSource:
@@ -86,7 +88,7 @@ class MediaSourceItem:
     @callback
     def async_media_source(self) -> MediaSource:
         """Return media source that owns this item."""
-        return self.hass.data[DOMAIN][self.domain]
+        return cast(MediaSource, self.hass.data[DOMAIN][self.domain])
 
     @classmethod
     def from_uri(cls, hass: HomeAssistant, uri: str) -> MediaSourceItem:
@@ -105,9 +107,9 @@ class MediaSourceItem:
 class MediaSource(ABC):
     """Represents a source of media files."""
 
-    name: str = None
+    name: str | None = None
 
-    def __init__(self, domain: str):
+    def __init__(self, domain: str) -> None:
         """Initialize a media source."""
         self.domain = domain
         if not self.name:
@@ -117,8 +119,6 @@ class MediaSource(ABC):
         """Resolve a media item to a playable item."""
         raise NotImplementedError
 
-    async def async_browse_media(
-        self, item: MediaSourceItem, media_types: Tuple[str]
-    ) -> BrowseMediaSource:
+    async def async_browse_media(self, item: MediaSourceItem) -> BrowseMediaSource:
         """Browse media."""
         raise NotImplementedError
