@@ -36,16 +36,9 @@ import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.reload import async_setup_reload_service
 from homeassistant.helpers.typing import ConfigType
 
-from . import (
-    CONF_COMMAND_TOPIC,
-    CONF_QOS,
-    CONF_RETAIN,
-    CONF_STATE_TOPIC,
-    DOMAIN,
-    PLATFORMS,
-    subscription,
-)
+from . import PLATFORMS, subscription
 from .. import mqtt
+from .const import CONF_COMMAND_TOPIC, CONF_QOS, CONF_RETAIN, CONF_STATE_TOPIC, DOMAIN
 from .debug_info import log_messages
 from .mixins import MQTT_ENTITY_COMMON_SCHEMA, MqttEntity, async_setup_entry_helper
 
@@ -101,6 +94,13 @@ TILT_FEATURES = (
     | SUPPORT_SET_TILT_POSITION
 )
 
+MQTT_COVER_ATTRIBUTES_BLOCKED = frozenset(
+    {
+        cover.ATTR_CURRENT_POSITION,
+        cover.ATTR_CURRENT_TILT_POSITION,
+    }
+)
+
 
 def validate_options(value):
     """Validate options.
@@ -142,53 +142,58 @@ def validate_options(value):
     return value
 
 
+_PLATFORM_SCHEMA_BASE = mqtt.MQTT_BASE_PLATFORM_SCHEMA.extend(
+    {
+        vol.Optional(CONF_COMMAND_TOPIC): mqtt.valid_publish_topic,
+        vol.Optional(CONF_DEVICE_CLASS): DEVICE_CLASSES_SCHEMA,
+        vol.Optional(CONF_GET_POSITION_TOPIC): mqtt.valid_subscribe_topic,
+        vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
+        vol.Optional(CONF_OPTIMISTIC, default=DEFAULT_OPTIMISTIC): cv.boolean,
+        vol.Optional(CONF_PAYLOAD_CLOSE, default=DEFAULT_PAYLOAD_CLOSE): vol.Any(
+            cv.string, None
+        ),
+        vol.Optional(CONF_PAYLOAD_OPEN, default=DEFAULT_PAYLOAD_OPEN): vol.Any(
+            cv.string, None
+        ),
+        vol.Optional(CONF_PAYLOAD_STOP, default=DEFAULT_PAYLOAD_STOP): vol.Any(
+            cv.string, None
+        ),
+        vol.Optional(CONF_POSITION_CLOSED, default=DEFAULT_POSITION_CLOSED): int,
+        vol.Optional(CONF_POSITION_OPEN, default=DEFAULT_POSITION_OPEN): int,
+        vol.Optional(CONF_RETAIN, default=DEFAULT_RETAIN): cv.boolean,
+        vol.Optional(CONF_SET_POSITION_TEMPLATE): cv.template,
+        vol.Optional(CONF_SET_POSITION_TOPIC): mqtt.valid_publish_topic,
+        vol.Optional(CONF_STATE_CLOSED, default=STATE_CLOSED): cv.string,
+        vol.Optional(CONF_STATE_CLOSING, default=STATE_CLOSING): cv.string,
+        vol.Optional(CONF_STATE_OPEN, default=STATE_OPEN): cv.string,
+        vol.Optional(CONF_STATE_OPENING, default=STATE_OPENING): cv.string,
+        vol.Optional(CONF_STATE_STOPPED, default=DEFAULT_STATE_STOPPED): cv.string,
+        vol.Optional(CONF_STATE_TOPIC): mqtt.valid_subscribe_topic,
+        vol.Optional(
+            CONF_TILT_CLOSED_POSITION, default=DEFAULT_TILT_CLOSED_POSITION
+        ): int,
+        vol.Optional(CONF_TILT_COMMAND_TOPIC): mqtt.valid_publish_topic,
+        vol.Optional(CONF_TILT_MAX, default=DEFAULT_TILT_MAX): int,
+        vol.Optional(CONF_TILT_MIN, default=DEFAULT_TILT_MIN): int,
+        vol.Optional(CONF_TILT_OPEN_POSITION, default=DEFAULT_TILT_OPEN_POSITION): int,
+        vol.Optional(
+            CONF_TILT_STATE_OPTIMISTIC, default=DEFAULT_TILT_OPTIMISTIC
+        ): cv.boolean,
+        vol.Optional(CONF_TILT_STATUS_TOPIC): mqtt.valid_subscribe_topic,
+        vol.Optional(CONF_TILT_STATUS_TEMPLATE): cv.template,
+        vol.Optional(CONF_VALUE_TEMPLATE): cv.template,
+        vol.Optional(CONF_GET_POSITION_TEMPLATE): cv.template,
+        vol.Optional(CONF_TILT_COMMAND_TEMPLATE): cv.template,
+    }
+).extend(MQTT_ENTITY_COMMON_SCHEMA.schema)
+
 PLATFORM_SCHEMA = vol.All(
-    mqtt.MQTT_BASE_PLATFORM_SCHEMA.extend(
-        {
-            vol.Optional(CONF_COMMAND_TOPIC): mqtt.valid_publish_topic,
-            vol.Optional(CONF_DEVICE_CLASS): DEVICE_CLASSES_SCHEMA,
-            vol.Optional(CONF_GET_POSITION_TOPIC): mqtt.valid_subscribe_topic,
-            vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
-            vol.Optional(CONF_OPTIMISTIC, default=DEFAULT_OPTIMISTIC): cv.boolean,
-            vol.Optional(CONF_PAYLOAD_CLOSE, default=DEFAULT_PAYLOAD_CLOSE): vol.Any(
-                cv.string, None
-            ),
-            vol.Optional(CONF_PAYLOAD_OPEN, default=DEFAULT_PAYLOAD_OPEN): vol.Any(
-                cv.string, None
-            ),
-            vol.Optional(CONF_PAYLOAD_STOP, default=DEFAULT_PAYLOAD_STOP): vol.Any(
-                cv.string, None
-            ),
-            vol.Optional(CONF_POSITION_CLOSED, default=DEFAULT_POSITION_CLOSED): int,
-            vol.Optional(CONF_POSITION_OPEN, default=DEFAULT_POSITION_OPEN): int,
-            vol.Optional(CONF_RETAIN, default=DEFAULT_RETAIN): cv.boolean,
-            vol.Optional(CONF_SET_POSITION_TEMPLATE): cv.template,
-            vol.Optional(CONF_SET_POSITION_TOPIC): mqtt.valid_publish_topic,
-            vol.Optional(CONF_STATE_CLOSED, default=STATE_CLOSED): cv.string,
-            vol.Optional(CONF_STATE_CLOSING, default=STATE_CLOSING): cv.string,
-            vol.Optional(CONF_STATE_OPEN, default=STATE_OPEN): cv.string,
-            vol.Optional(CONF_STATE_OPENING, default=STATE_OPENING): cv.string,
-            vol.Optional(CONF_STATE_STOPPED, default=DEFAULT_STATE_STOPPED): cv.string,
-            vol.Optional(CONF_STATE_TOPIC): mqtt.valid_subscribe_topic,
-            vol.Optional(
-                CONF_TILT_CLOSED_POSITION, default=DEFAULT_TILT_CLOSED_POSITION
-            ): int,
-            vol.Optional(CONF_TILT_COMMAND_TOPIC): mqtt.valid_publish_topic,
-            vol.Optional(CONF_TILT_MAX, default=DEFAULT_TILT_MAX): int,
-            vol.Optional(CONF_TILT_MIN, default=DEFAULT_TILT_MIN): int,
-            vol.Optional(
-                CONF_TILT_OPEN_POSITION, default=DEFAULT_TILT_OPEN_POSITION
-            ): int,
-            vol.Optional(
-                CONF_TILT_STATE_OPTIMISTIC, default=DEFAULT_TILT_OPTIMISTIC
-            ): cv.boolean,
-            vol.Optional(CONF_TILT_STATUS_TOPIC): mqtt.valid_subscribe_topic,
-            vol.Optional(CONF_TILT_STATUS_TEMPLATE): cv.template,
-            vol.Optional(CONF_VALUE_TEMPLATE): cv.template,
-            vol.Optional(CONF_GET_POSITION_TEMPLATE): cv.template,
-            vol.Optional(CONF_TILT_COMMAND_TEMPLATE): cv.template,
-        }
-    ).extend(MQTT_ENTITY_COMMON_SCHEMA.schema),
+    _PLATFORM_SCHEMA_BASE,
+    validate_options,
+)
+
+DISCOVERY_SCHEMA = vol.All(
+    _PLATFORM_SCHEMA_BASE.extend({}, extra=vol.REMOVE_EXTRA),
     validate_options,
 )
 
@@ -206,7 +211,7 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
     setup = functools.partial(
         _async_setup_entity, hass, async_add_entities, config_entry=config_entry
     )
-    await async_setup_entry_helper(hass, cover.DOMAIN, setup, PLATFORM_SCHEMA)
+    await async_setup_entry_helper(hass, cover.DOMAIN, setup, DISCOVERY_SCHEMA)
 
 
 async def _async_setup_entity(
@@ -219,6 +224,8 @@ async def _async_setup_entity(
 class MqttCover(MqttEntity, CoverEntity):
     """Representation of a cover that can be controlled using MQTT."""
 
+    _attributes_extra_blocked = MQTT_COVER_ATTRIBUTES_BLOCKED
+
     def __init__(self, hass, config, config_entry, discovery_data):
         """Initialize the cover."""
         self._position = None
@@ -233,14 +240,48 @@ class MqttCover(MqttEntity, CoverEntity):
     @staticmethod
     def config_schema():
         """Return the config schema."""
-        return PLATFORM_SCHEMA
+        return DISCOVERY_SCHEMA
 
     def _setup_from_config(self, config):
-        self._optimistic = config[CONF_OPTIMISTIC] or (
-            config.get(CONF_STATE_TOPIC) is None
+        no_position = (
+            config.get(CONF_SET_POSITION_TOPIC) is None
             and config.get(CONF_GET_POSITION_TOPIC) is None
         )
-        self._tilt_optimistic = config[CONF_TILT_STATE_OPTIMISTIC]
+        no_state = (
+            config.get(CONF_COMMAND_TOPIC) is None
+            and config.get(CONF_STATE_TOPIC) is None
+        )
+        no_tilt = (
+            config.get(CONF_TILT_COMMAND_TOPIC) is None
+            and config.get(CONF_TILT_STATUS_TOPIC) is None
+        )
+        optimistic_position = (
+            config.get(CONF_SET_POSITION_TOPIC) is not None
+            and config.get(CONF_GET_POSITION_TOPIC) is None
+        )
+        optimistic_state = (
+            config.get(CONF_COMMAND_TOPIC) is not None
+            and config.get(CONF_STATE_TOPIC) is None
+        )
+        optimistic_tilt = (
+            config.get(CONF_TILT_COMMAND_TOPIC) is not None
+            and config.get(CONF_TILT_STATUS_TOPIC) is None
+        )
+
+        if config[CONF_OPTIMISTIC] or (
+            (no_position or optimistic_position)
+            and (no_state or optimistic_state)
+            and (no_tilt or optimistic_tilt)
+        ):
+            # Force into optimistic mode.
+            self._optimistic = True
+
+        if (
+            config[CONF_TILT_STATE_OPTIMISTIC]
+            or config.get(CONF_TILT_STATUS_TOPIC) is None
+        ):
+            # Force into optimistic tilt mode.
+            self._tilt_optimistic = True
 
         value_template = self._config.get(CONF_VALUE_TEMPLATE)
         if value_template is not None:
@@ -409,17 +450,7 @@ class MqttCover(MqttEntity, CoverEntity):
                 "qos": self._config[CONF_QOS],
             }
 
-        if (
-            self._config.get(CONF_GET_POSITION_TOPIC) is None
-            and self._config.get(CONF_STATE_TOPIC) is None
-        ):
-            # Force into optimistic mode.
-            self._optimistic = True
-
-        if self._config.get(CONF_TILT_STATUS_TOPIC) is None:
-            # Force into optimistic tilt mode.
-            self._tilt_optimistic = True
-        else:
+        if self._config.get(CONF_TILT_STATUS_TOPIC) is not None:
             self._tilt_value = STATE_UNKNOWN
             topics["tilt_status_topic"] = {
                 "topic": self._config.get(CONF_TILT_STATUS_TOPIC),
