@@ -1,10 +1,18 @@
 """Base class for Netatmo entities."""
 from __future__ import annotations
 
-from homeassistant.core import CALLBACK_TYPE, callback
-from homeassistant.helpers.entity import Entity
+from homeassistant.const import ATTR_ATTRIBUTION
+from homeassistant.core import callback
+from homeassistant.helpers.entity import DeviceInfo, Entity
 
-from .const import DATA_DEVICE_IDS, DOMAIN, MANUFACTURER, MODELS, SIGNAL_NAME
+from .const import (
+    DATA_DEVICE_IDS,
+    DEFAULT_ATTRIBUTION,
+    DOMAIN,
+    MANUFACTURER,
+    MODELS,
+    SIGNAL_NAME,
+)
 from .data_handler import PUBLICDATA_DATA_CLASS_NAME, NetatmoDataHandler
 
 
@@ -15,13 +23,14 @@ class NetatmoBase(Entity):
         """Set up Netatmo entity base."""
         self.data_handler = data_handler
         self._data_classes: list[dict] = []
-        self._listeners: list[CALLBACK_TYPE] = []
 
-        self._device_name = None
-        self._id = None
-        self._model = None
-        self._name = None
-        self._unique_id = None
+        self._device_name: str = ""
+        self._id: str = ""
+        self._model: str = ""
+        self._netatmo_type: str = ""
+        self._attr_name = None
+        self._attr_unique_id = None
+        self._attr_extra_state_attributes = {ATTR_ATTRIBUTION: DEFAULT_ATTRIBUTION}
 
     async def async_added_to_hass(self) -> None:
         """Entity created."""
@@ -52,7 +61,7 @@ class NetatmoBase(Entity):
                     data_class["name"], signal_name, self.async_update_callback
                 )
 
-            for sub in self.data_handler.data_classes[signal_name].get("subscriptions"):
+            for sub in self.data_handler.data_classes[signal_name].subscriptions:
                 if sub is None:
                     await self.data_handler.unregister_data_class(signal_name, None)
 
@@ -62,44 +71,27 @@ class NetatmoBase(Entity):
 
         self.async_update_callback()
 
-    async def async_will_remove_from_hass(self):
+    async def async_will_remove_from_hass(self) -> None:
         """Run when entity will be removed from hass."""
         await super().async_will_remove_from_hass()
 
-        for listener in self._listeners:
-            listener()
-
         for data_class in self._data_classes:
             await self.data_handler.unregister_data_class(
                 data_class[SIGNAL_NAME], self.async_update_callback
             )
 
     @callback
-    def async_update_callback(self):
+    def async_update_callback(self) -> None:
         """Update the entity's state."""
         raise NotImplementedError
 
     @property
-    def _data(self):
-        """Return data for this entity."""
-        return self.data_handler.data[self._data_classes[0]["name"]]
-
-    @property
-    def unique_id(self):
-        """Return the unique ID of this entity."""
-        return self._unique_id
-
-    @property
-    def name(self):
-        """Return the name of this entity."""
-        return self._name
-
-    @property
-    def device_info(self):
+    def device_info(self) -> DeviceInfo:
         """Return the device info for the sensor."""
-        return {
-            "identifiers": {(DOMAIN, self._id)},
-            "name": self._device_name,
-            "manufacturer": MANUFACTURER,
-            "model": MODELS[self._model],
-        }
+        return DeviceInfo(
+            configuration_url=f"https://my.netatmo.com/app/{self._netatmo_type}",
+            identifiers={(DOMAIN, self._id)},
+            name=self._device_name,
+            manufacturer=MANUFACTURER,
+            model=MODELS[self._model],
+        )
