@@ -1,8 +1,10 @@
 """Schema migration helpers."""
+import contextlib
+from datetime import timedelta
 import logging
 
 import sqlalchemy
-from sqlalchemy import ForeignKeyConstraint, MetaData, Table, text
+from sqlalchemy import ForeignKeyConstraint, MetaData, Table, func, text
 from sqlalchemy.exc import (
     InternalError,
     OperationalError,
@@ -10,8 +12,20 @@ from sqlalchemy.exc import (
     SQLAlchemyError,
 )
 from sqlalchemy.schema import AddConstraint, DropConstraint
+from sqlalchemy.sql.expression import true
 
-from .models import SCHEMA_VERSION, TABLE_STATES, Base, SchemaChanges, Statistics
+from .models import (
+    SCHEMA_VERSION,
+    TABLE_STATES,
+    Base,
+    SchemaChanges,
+    Statistics,
+    StatisticsMeta,
+    StatisticsRuns,
+    StatisticsShortTerm,
+    process_timestamp,
+)
+from .statistics import get_start_time
 from .util import session_scope
 
 _LOGGER = logging.getLogger(__name__)
@@ -61,7 +75,7 @@ def migrate_schema(instance, current_version):
         for version in range(current_version, SCHEMA_VERSION):
             new_version = version + 1
             _LOGGER.info("Upgrading recorder db schema to version %s", new_version)
-            _apply_update(instance.engine, session, new_version, current_version)
+            _apply_update(instance, session, new_version, current_version)
             session.add(SchemaChanges(schema_version=new_version))
 
             _LOGGER.info("Upgrade to version %s done", new_version)
@@ -297,7 +311,7 @@ def _update_states_table_with_foreign_key_options(connection, engine):
 
     states_key_constraints = Base.metadata.tables[TABLE_STATES].foreign_key_constraints
     old_states_table = Table(  # noqa: F841 pylint: disable=unused-variable
-        TABLE_STATES, MetaData(), *[alter["old_fk"] for alter in alters]
+        TABLE_STATES, MetaData(), *(alter["old_fk"] for alter in alters)
     )
 
     for alter in alters:
@@ -340,8 +354,9 @@ def _drop_foreign_key_constraints(connection, engine, table, columns):
             )
 
 
-def _apply_update(engine, session, new_version, old_version):
+def _apply_update(instance, session, new_version, old_version):  # noqa: C901
     """Perform operations to bring schema up to date."""
+    engine = instance.engine
     connection = session.connection()
     if new_version == 1:
         _create_index(connection, "events", "ix_events_time_fired")
@@ -444,14 +459,135 @@ def _apply_update(engine, session, new_version, old_version):
     elif new_version == 14:
         _modify_columns(connection, engine, "events", ["event_type VARCHAR(64)"])
     elif new_version == 15:
-        if sqlalchemy.inspect(engine).has_table(Statistics.__tablename__):
-            # Recreate the statistics table
-            Statistics.__table__.drop(engine)
-            Statistics.__table__.create(engine)
+        # This dropped the statistics table, done again in version 18.
+        pass
     elif new_version == 16:
         _drop_foreign_key_constraints(
             connection, engine, TABLE_STATES, ["old_state_id"]
         )
+    elif new_version == 17:
+        # This dropped the statistics table, done again in version 18.
+        pass
+    elif new_version == 18:
+        # Recreate the statistics and statistics meta tables.
+        #
+        # Order matters! Statistics and StatisticsShortTerm have a relation with
+        # StatisticsMeta, so statistics need to be deleted before meta (or in pair
+        # depending on the SQL backend); and meta needs to be created before statistics.
+        Base.metadata.drop_all(
+            bind=engine,
+            tables=[
+                StatisticsShortTerm.__table__,
+                Statistics.__table__,
+                StatisticsMeta.__table__,
+            ],
+        )
+
+        StatisticsMeta.__table__.create(engine)
+        StatisticsShortTerm.__table__.create(engine)
+        Statistics.__table__.create(engine)
+    elif new_version == 19:
+        # This adds the statistic runs table, insert a fake run to prevent duplicating
+        # statistics.
+        session.add(StatisticsRuns(start=get_start_time()))
+    elif new_version == 20:
+        # This changed the precision of statistics from float to double
+        if engine.dialect.name in ["mysql", "postgresql"]:
+            _modify_columns(
+                connection,
+                engine,
+                "statistics",
+                [
+                    "mean DOUBLE PRECISION",
+                    "min DOUBLE PRECISION",
+                    "max DOUBLE PRECISION",
+                    "state DOUBLE PRECISION",
+                    "sum DOUBLE PRECISION",
+                ],
+            )
+    elif new_version == 21:
+        # Try to change the character set of the statistic_meta table
+        if engine.dialect.name == "mysql":
+            for table in ("events", "states", "statistics_meta"):
+                _LOGGER.warning(
+                    "Updating character set and collation of table %s to utf8mb4. "
+                    "Note: this can take several minutes on large databases and slow "
+                    "computers. Please be patient!",
+                    table,
+                )
+                with contextlib.suppress(SQLAlchemyError):
+                    connection.execute(
+                        # Using LOCK=EXCLUSIVE to prevent the database from corrupting
+                        # https://github.com/home-assistant/core/issues/56104
+                        text(
+                            f"ALTER TABLE {table} CONVERT TO "
+                            "CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci LOCK=EXCLUSIVE"
+                        )
+                    )
+    elif new_version == 22:
+        # Recreate the all statistics tables for Oracle DB with Identity columns
+        #
+        # Order matters! Statistics has a relation with StatisticsMeta,
+        # so statistics need to be deleted before meta (or in pair depending
+        # on the SQL backend); and meta needs to be created before statistics.
+        if engine.dialect.name == "oracle":
+            Base.metadata.drop_all(
+                bind=engine,
+                tables=[
+                    StatisticsShortTerm.__table__,
+                    Statistics.__table__,
+                    StatisticsMeta.__table__,
+                    StatisticsRuns.__table__,
+                ],
+            )
+
+            StatisticsRuns.__table__.create(engine)
+            StatisticsMeta.__table__.create(engine)
+            StatisticsShortTerm.__table__.create(engine)
+            Statistics.__table__.create(engine)
+
+        # Block 5-minute statistics for one hour from the last run, or it will overlap
+        # with existing hourly statistics. Don't block on a database with no existing
+        # statistics.
+        if session.query(Statistics.id).count() and (
+            last_run_string := session.query(func.max(StatisticsRuns.start)).scalar()
+        ):
+            last_run_start_time = process_timestamp(last_run_string)
+            if last_run_start_time:
+                fake_start_time = last_run_start_time + timedelta(minutes=5)
+                while fake_start_time < last_run_start_time + timedelta(hours=1):
+                    session.add(StatisticsRuns(start=fake_start_time))
+                    fake_start_time += timedelta(minutes=5)
+
+        # When querying the database, be careful to only explicitly query for columns
+        # which were present in schema version 21. If querying the table, SQLAlchemy
+        # will refer to future columns.
+        for sum_statistic in session.query(StatisticsMeta.id).filter_by(has_sum=true()):
+            last_statistic = (
+                session.query(
+                    Statistics.start,
+                    Statistics.last_reset,
+                    Statistics.state,
+                    Statistics.sum,
+                )
+                .filter_by(metadata_id=sum_statistic.id)
+                .order_by(Statistics.start.desc())
+                .first()
+            )
+            if last_statistic:
+                session.add(
+                    StatisticsShortTerm(
+                        metadata_id=sum_statistic.id,
+                        start=last_statistic.start,
+                        last_reset=last_statistic.last_reset,
+                        state=last_statistic.state,
+                        sum=last_statistic.sum,
+                    )
+                )
+    elif new_version == 23:
+        # Add name column to StatisticsMeta
+        _add_columns(session, "statistics_meta", ["name VARCHAR(255)"])
+
     else:
         raise ValueError(f"No schema migration defined for version {new_version}")
 
@@ -471,6 +607,7 @@ def _inspect_schema_version(engine, session):
     for index in indexes:
         if index["column_names"] == ["time_fired"]:
             # Schema addition from version 1 detected. New DB.
+            session.add(StatisticsRuns(start=get_start_time()))
             session.add(SchemaChanges(schema_version=SCHEMA_VERSION))
             return SCHEMA_VERSION
 
