@@ -1,22 +1,19 @@
 """Entity representing a Sonos power sensor."""
 from __future__ import annotations
 
-import datetime
-import logging
 from typing import Any
 
 from homeassistant.components.binary_sensor import (
     DEVICE_CLASS_BATTERY_CHARGING,
     BinarySensorEntity,
 )
+from homeassistant.const import ENTITY_CATEGORY_DIAGNOSTIC
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
 
-from .const import DATA_SONOS, SONOS_CREATE_BATTERY
-from .entity import SonosSensorEntity
+from .const import SONOS_CREATE_BATTERY
+from .entity import SonosEntity
 from .speaker import SonosSpeaker
 
-_LOGGER = logging.getLogger(__name__)
-
 ATTR_BATTERY_POWER_SOURCE = "power_source"
 
 
@@ -24,7 +21,7 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
     """Set up Sonos from a config entry."""
 
     async def _async_create_entity(speaker: SonosSpeaker) -> None:
-        entity = SonosPowerEntity(speaker, hass.data[DATA_SONOS])
+        entity = SonosPowerEntity(speaker)
         async_add_entities([entity])
 
     config_entry.async_on_unload(
@@ -32,9 +29,11 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
     )
 
 
-class SonosPowerEntity(SonosSensorEntity, BinarySensorEntity):
+class SonosPowerEntity(SonosEntity, BinarySensorEntity):
     """Representation of a Sonos power entity."""
 
+    _attr_entity_category = ENTITY_CATEGORY_DIAGNOSTIC
+
     @property
     def unique_id(self) -> str:
         """Return the unique ID of the sensor."""
@@ -50,7 +49,7 @@ class SonosPowerEntity(SonosSensorEntity, BinarySensorEntity):
         """Return the entity's device class."""
         return DEVICE_CLASS_BATTERY_CHARGING
 
-    async def async_update(self, now: datetime.datetime | None = None) -> None:
+    async def _async_poll(self) -> None:
         """Poll the device for the current state."""
         await self.speaker.async_poll_battery()
 
@@ -65,3 +64,8 @@ class SonosPowerEntity(SonosSensorEntity, BinarySensorEntity):
         return {
             ATTR_BATTERY_POWER_SOURCE: self.speaker.power_source,
         }
+
+    @property
+    def available(self) -> bool:
+        """Return whether this device is available."""
+        return self.speaker.available and (self.speaker.charging is not None)
