@@ -1,11 +1,11 @@
 """Config flow for the Total Connect component."""
-from total_connect_client import TotalConnectClient
+from total_connect_client.client import TotalConnectClient
 import voluptuous as vol
 
 from homeassistant import config_entries
 from homeassistant.const import CONF_PASSWORD, CONF_USERNAME
 
-from .const import CONF_USERCODES, DOMAIN  # pylint: disable=unused-import
+from .const import CONF_USERCODES, DOMAIN
 
 CONF_LOCATION = "location"
 
@@ -37,7 +37,7 @@ class TotalConnectConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
             self._abort_if_unique_id_configured()
 
             client = await self.hass.async_add_executor_job(
-                TotalConnectClient.TotalConnectClient, username, password, None
+                TotalConnectClient, username, password, None
             )
 
             if client.is_valid_credentials():
@@ -65,10 +65,10 @@ class TotalConnectConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
                 if self.usercodes[location_id] is None:
                     valid = await self.hass.async_add_executor_job(
                         self.client.locations[location_id].set_usercode,
-                        user_entry[CONF_LOCATION],
+                        user_entry[CONF_USERCODES],
                     )
                     if valid:
-                        self.usercodes[location_id] = user_entry[CONF_LOCATION]
+                        self.usercodes[location_id] = user_entry[CONF_USERCODES]
                     else:
                         errors[CONF_LOCATION] = "usercode"
                     break
@@ -93,12 +93,14 @@ class TotalConnectConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
 
         # show the next location that needs a usercode
         location_codes = {}
+        location_for_user = ""
         for location_id in self.usercodes:
             if self.usercodes[location_id] is None:
+                location_for_user = location_id
                 location_codes[
                     vol.Required(
-                        CONF_LOCATION,
-                        default=location_id,
+                        CONF_USERCODES,
+                        default="0000",
                     )
                 ] = str
                 break
@@ -108,7 +110,7 @@ class TotalConnectConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
             step_id="locations",
             data_schema=data_schema,
             errors=errors,
-            description_placeholders={"base": "description"},
+            description_placeholders={"location_id": location_for_user},
         )
 
     async def async_step_reauth(self, config):
@@ -128,7 +130,7 @@ class TotalConnectConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
             )
 
         client = await self.hass.async_add_executor_job(
-            TotalConnectClient.TotalConnectClient,
+            TotalConnectClient,
             self.username,
             user_input[CONF_PASSWORD],
             self.usercodes,
