@@ -1,25 +1,29 @@
 """Helpers for the data entry flow."""
+from __future__ import annotations
 
+from http import HTTPStatus
+from typing import Any
+
+from aiohttp import web
 import voluptuous as vol
 
 from homeassistant import config_entries, data_entry_flow
 from homeassistant.components.http import HomeAssistantView
 from homeassistant.components.http.data_validator import RequestDataValidator
-from homeassistant.const import HTTP_NOT_FOUND
 import homeassistant.helpers.config_validation as cv
 
-# mypy: allow-untyped-calls, allow-untyped-defs
-
 
 class _BaseFlowManagerView(HomeAssistantView):
     """Foundation for flow manager views."""
 
-    def __init__(self, flow_mgr):
+    def __init__(self, flow_mgr: data_entry_flow.FlowManager) -> None:
         """Initialize the flow manager index view."""
         self._flow_mgr = flow_mgr
 
     # pylint: disable=no-self-use
-    def _prepare_result_json(self, result):
+    def _prepare_result_json(
+        self, result: data_entry_flow.FlowResult
+    ) -> data_entry_flow.FlowResult:
         """Convert result to JSON."""
         if result["type"] == data_entry_flow.RESULT_TYPE_CREATE_ENTRY:
             data = result.copy()
@@ -34,8 +38,7 @@ class _BaseFlowManagerView(HomeAssistantView):
 
         data = result.copy()
 
-        schema = data["data_schema"]
-        if schema is None:
+        if (schema := data["data_schema"]) is None:
             data["data_schema"] = []
         else:
             data["data_schema"] = voluptuous_serialize.convert(
@@ -57,7 +60,7 @@ class FlowManagerIndexView(_BaseFlowManagerView):
             extra=vol.ALLOW_EXTRA,
         )
     )
-    async def post(self, request, data):
+    async def post(self, request: web.Request, data: dict[str, Any]) -> web.Response:
         """Handle a POST request."""
         if isinstance(data["handler"], list):
             handler = tuple(data["handler"])
@@ -66,16 +69,18 @@ class FlowManagerIndexView(_BaseFlowManagerView):
 
         try:
             result = await self._flow_mgr.async_init(
-                handler,
+                handler,  # type: ignore
                 context={
                     "source": config_entries.SOURCE_USER,
                     "show_advanced_options": data["show_advanced_options"],
                 },
             )
         except data_entry_flow.UnknownHandler:
-            return self.json_message("Invalid handler specified", HTTP_NOT_FOUND)
+            return self.json_message("Invalid handler specified", HTTPStatus.NOT_FOUND)
         except data_entry_flow.UnknownStep:
-            return self.json_message("Handler does not support user", 400)
+            return self.json_message(
+                "Handler does not support user", HTTPStatus.BAD_REQUEST
+            )
 
         result = self._prepare_result_json(result)
 
@@ -85,36 +90,38 @@ class FlowManagerIndexView(_BaseFlowManagerView):
 class FlowManagerResourceView(_BaseFlowManagerView):
     """View to interact with the flow manager."""
 
-    async def get(self, request, flow_id):
+    async def get(self, request: web.Request, flow_id: str) -> web.Response:
         """Get the current state of a data_entry_flow."""
         try:
             result = await self._flow_mgr.async_configure(flow_id)
         except data_entry_flow.UnknownFlow:
-            return self.json_message("Invalid flow specified", HTTP_NOT_FOUND)
+            return self.json_message("Invalid flow specified", HTTPStatus.NOT_FOUND)
 
         result = self._prepare_result_json(result)
 
         return self.json(result)
 
     @RequestDataValidator(vol.Schema(dict), allow_empty=True)
-    async def post(self, request, flow_id, data):
+    async def post(
+        self, request: web.Request, flow_id: str, data: dict[str, Any]
+    ) -> web.Response:
         """Handle a POST request."""
         try:
             result = await self._flow_mgr.async_configure(flow_id, data)
         except data_entry_flow.UnknownFlow:
-            return self.json_message("Invalid flow specified", HTTP_NOT_FOUND)
+            return self.json_message("Invalid flow specified", HTTPStatus.NOT_FOUND)
         except vol.Invalid:
-            return self.json_message("User input malformed", 400)
+            return self.json_message("User input malformed", HTTPStatus.BAD_REQUEST)
 
         result = self._prepare_result_json(result)
 
         return self.json(result)
 
-    async def delete(self, request, flow_id):
+    async def delete(self, request: web.Request, flow_id: str) -> web.Response:
         """Cancel a flow in progress."""
         try:
             self._flow_mgr.async_abort(flow_id)
         except data_entry_flow.UnknownFlow:
-            return self.json_message("Invalid flow specified", HTTP_NOT_FOUND)
+            return self.json_message("Invalid flow specified", HTTPStatus.NOT_FOUND)
 
         return self.json_message("Flow aborted")
