@@ -3,18 +3,22 @@ from __future__ import annotations
 
 from abc import ABC
 import asyncio
-from collections.abc import Awaitable, Iterable, Mapping
+from collections.abc import Awaitable, Iterable, Mapping, MutableMapping
+from dataclasses import dataclass
 from datetime import datetime, timedelta
 import functools as ft
 import logging
 import math
 import sys
 from timeit import default_timer as timer
-from typing import Any
+from typing import Any, Final, Literal, TypedDict, final
+
+import voluptuous as vol
 
 from homeassistant.config import DATA_CUSTOMIZE
 from homeassistant.const import (
     ATTR_ASSUMED_STATE,
+    ATTR_ATTRIBUTION,
     ATTR_DEVICE_CLASS,
     ATTR_ENTITY_PICTURE,
     ATTR_FRIENDLY_NAME,
@@ -22,6 +26,8 @@ from homeassistant.const import (
     ATTR_SUPPORTED_FEATURES,
     ATTR_UNIT_OF_MEASUREMENT,
     DEVICE_DEFAULT_NAME,
+    ENTITY_CATEGORY_CONFIG,
+    ENTITY_CATEGORY_DIAGNOSTIC,
     STATE_OFF,
     STATE_ON,
     STATE_UNAVAILABLE,
@@ -33,7 +39,6 @@ from homeassistant.core import CALLBACK_TYPE, Context, HomeAssistant, callback
 from homeassistant.exceptions import HomeAssistantError, NoEntitySpecifiedError
 from homeassistant.helpers import entity_registry as er
 from homeassistant.helpers.entity_platform import EntityPlatform
-from homeassistant.helpers.entity_registry import RegistryEntry
 from homeassistant.helpers.event import Event, async_track_entity_registry_updated_event
 from homeassistant.helpers.typing import StateType
 from homeassistant.loader import bind_hass
@@ -50,6 +55,14 @@ SOURCE_PLATFORM_CONFIG = "platform_config"
 FLOAT_PRECISION = abs(int(math.floor(math.log10(abs(sys.float_info.epsilon))))) - 1
 
 
+ENTITY_CATEGORIES: Final[list[str]] = [
+    ENTITY_CATEGORY_CONFIG,
+    ENTITY_CATEGORY_DIAGNOSTIC,
+]
+
+ENTITY_CATEGORIES_SCHEMA: Final = vol.In(ENTITY_CATEGORIES)
+
+
 @callback
 @bind_hass
 def entity_sources(hass: HomeAssistant) -> dict[str, dict[str, str]]:
@@ -93,23 +106,100 @@ def async_generate_entity_id(
     return test_string
 
 
+def get_capability(hass: HomeAssistant, entity_id: str, capability: str) -> Any | None:
+    """Get a capability attribute of an entity.
+
+    First try the statemachine, then entity registry.
+    """
+    if state := hass.states.get(entity_id):
+        return state.attributes.get(capability)
+
+    entity_registry = er.async_get(hass)
+    if not (entry := entity_registry.async_get(entity_id)):
+        raise HomeAssistantError(f"Unknown entity {entity_id}")
+
+    return entry.capabilities.get(capability) if entry.capabilities else None
+
+
+def get_device_class(hass: HomeAssistant, entity_id: str) -> str | None:
+    """Get device class of an entity.
+
+    First try the statemachine, then entity registry.
+    """
+    if state := hass.states.get(entity_id):
+        return state.attributes.get(ATTR_DEVICE_CLASS)
+
+    entity_registry = er.async_get(hass)
+    if not (entry := entity_registry.async_get(entity_id)):
+        raise HomeAssistantError(f"Unknown entity {entity_id}")
+
+    return entry.device_class
+
+
 def get_supported_features(hass: HomeAssistant, entity_id: str) -> int:
     """Get supported features for an entity.
 
     First try the statemachine, then entity registry.
     """
-    state = hass.states.get(entity_id)
-    if state:
+    if state := hass.states.get(entity_id):
         return state.attributes.get(ATTR_SUPPORTED_FEATURES, 0)
 
     entity_registry = er.async_get(hass)
-    entry = entity_registry.async_get(entity_id)
-    if not entry:
+    if not (entry := entity_registry.async_get(entity_id)):
         raise HomeAssistantError(f"Unknown entity {entity_id}")
 
     return entry.supported_features or 0
 
 
+def get_unit_of_measurement(hass: HomeAssistant, entity_id: str) -> str | None:
+    """Get unit of measurement class of an entity.
+
+    First try the statemachine, then entity registry.
+    """
+    if state := hass.states.get(entity_id):
+        return state.attributes.get(ATTR_UNIT_OF_MEASUREMENT)
+
+    entity_registry = er.async_get(hass)
+    if not (entry := entity_registry.async_get(entity_id)):
+        raise HomeAssistantError(f"Unknown entity {entity_id}")
+
+    return entry.unit_of_measurement
+
+
+class DeviceInfo(TypedDict, total=False):
+    """Entity device information for device registry."""
+
+    configuration_url: str | None
+    connections: set[tuple[str, str]]
+    default_manufacturer: str
+    default_model: str
+    default_name: str
+    entry_type: str | None
+    identifiers: set[tuple[str, str]]
+    manufacturer: str | None
+    model: str | None
+    name: str | None
+    suggested_area: str | None
+    sw_version: str | None
+    via_device: tuple[str, str]
+
+
+@dataclass
+class EntityDescription:
+    """A class that describes Home Assistant entities."""
+
+    # This is the key identifier for this entity
+    key: str
+
+    device_class: str | None = None
+    entity_category: Literal["config", "diagnostic"] | None = None
+    entity_registry_enabled_default: bool = True
+    force_update: bool = False
+    icon: str | None = None
+    name: str | None = None
+    unit_of_measurement: str | None = None
+
+
 class Entity(ABC):
     """An abstract class for Home Assistant entities."""
 
@@ -121,12 +211,14 @@ class Entity(ABC):
     # Owning hass instance. Will be set by EntityPlatform
     # While not purely typed, it makes typehinting more useful for us
     # and removes the need for constant None checks or asserts.
-    # Ignore types: https://github.com/PyCQA/pylint/issues/3167
     hass: HomeAssistant = None  # type: ignore
 
     # Owning platform instance. Will be set by EntityPlatform
     platform: EntityPlatform | None = None
 
+    # Entity description instance for this Entity
+    entity_description: EntityDescription
+
     # If we reported if this entity was slow
     _slow_reported = False
 
@@ -140,7 +232,7 @@ class Entity(ABC):
     parallel_updates: asyncio.Semaphore | None = None
 
     # Entry in the entity registry
-    registry_entry: RegistryEntry | None = None
+    registry_entry: er.RegistryEntry | None = None
 
     # Hold list for functions to call on remove.
     _on_remove: list[CALLBACK_TYPE] | None = None
@@ -152,28 +244,52 @@ class Entity(ABC):
     # If entity is added to an entity platform
     _added = False
 
+    # Entity Properties
+    _attr_assumed_state: bool = False
+    _attr_attribution: str | None = None
+    _attr_available: bool = True
+    _attr_context_recent_time: timedelta = timedelta(seconds=5)
+    _attr_device_class: str | None
+    _attr_device_info: DeviceInfo | None = None
+    _attr_entity_category: str | None
+    _attr_entity_picture: str | None = None
+    _attr_entity_registry_enabled_default: bool
+    _attr_extra_state_attributes: MutableMapping[str, Any]
+    _attr_force_update: bool
+    _attr_icon: str | None
+    _attr_name: str | None
+    _attr_should_poll: bool = True
+    _attr_state: StateType = STATE_UNKNOWN
+    _attr_supported_features: int | None = None
+    _attr_unique_id: str | None = None
+    _attr_unit_of_measurement: str | None
+
     @property
     def should_poll(self) -> bool:
         """Return True if entity has to be polled for state.
 
         False if entity pushes its state to HA.
         """
-        return True
+        return self._attr_should_poll
 
     @property
     def unique_id(self) -> str | None:
         """Return a unique ID."""
-        return None
+        return self._attr_unique_id
 
     @property
     def name(self) -> str | None:
         """Return the name of the entity."""
+        if hasattr(self, "_attr_name"):
+            return self._attr_name
+        if hasattr(self, "entity_description"):
+            return self.entity_description.name
         return None
 
     @property
     def state(self) -> StateType:
         """Return the state of the entity."""
-        return STATE_UNKNOWN
+        return self._attr_state
 
     @property
     def capability_attributes(self) -> Mapping[str, Any] | None:
@@ -211,45 +327,59 @@ class Entity(ABC):
         Implemented by platform classes. Convention for attribute names
         is lowercase snake_case.
         """
+        if hasattr(self, "_attr_extra_state_attributes"):
+            return self._attr_extra_state_attributes
         return None
 
     @property
-    def device_info(self) -> Mapping[str, Any] | None:
+    def device_info(self) -> DeviceInfo | None:
         """Return device specific attributes.
 
         Implemented by platform classes.
         """
-        return None
+        return self._attr_device_info
 
     @property
     def device_class(self) -> str | None:
         """Return the class of this device, from component DEVICE_CLASSES."""
+        if hasattr(self, "_attr_device_class"):
+            return self._attr_device_class
+        if hasattr(self, "entity_description"):
+            return self.entity_description.device_class
         return None
 
     @property
     def unit_of_measurement(self) -> str | None:
         """Return the unit of measurement of this entity, if any."""
+        if hasattr(self, "_attr_unit_of_measurement"):
+            return self._attr_unit_of_measurement
+        if hasattr(self, "entity_description"):
+            return self.entity_description.unit_of_measurement
         return None
 
     @property
     def icon(self) -> str | None:
         """Return the icon to use in the frontend, if any."""
+        if hasattr(self, "_attr_icon"):
+            return self._attr_icon
+        if hasattr(self, "entity_description"):
+            return self.entity_description.icon
         return None
 
     @property
     def entity_picture(self) -> str | None:
         """Return the entity picture to use in the frontend, if any."""
-        return None
+        return self._attr_entity_picture
 
     @property
     def available(self) -> bool:
         """Return True if entity is available."""
-        return True
+        return self._attr_available
 
     @property
     def assumed_state(self) -> bool:
         """Return True if unable to access real state of the entity."""
-        return False
+        return self._attr_assumed_state
 
     @property
     def force_update(self) -> bool:
@@ -258,23 +388,45 @@ class Entity(ABC):
         If True, a state change will be triggered anytime the state property is
         updated, not just when the value changes.
         """
+        if hasattr(self, "_attr_force_update"):
+            return self._attr_force_update
+        if hasattr(self, "entity_description"):
+            return self.entity_description.force_update
         return False
 
     @property
     def supported_features(self) -> int | None:
         """Flag supported features."""
-        return None
+        return self._attr_supported_features
 
     @property
     def context_recent_time(self) -> timedelta:
         """Time that a context is considered recent."""
-        return timedelta(seconds=5)
+        return self._attr_context_recent_time
 
     @property
     def entity_registry_enabled_default(self) -> bool:
         """Return if the entity should be enabled when first added to the entity registry."""
+        if hasattr(self, "_attr_entity_registry_enabled_default"):
+            return self._attr_entity_registry_enabled_default
+        if hasattr(self, "entity_description"):
+            return self.entity_description.entity_registry_enabled_default
         return True
 
+    @property
+    def attribution(self) -> str | None:
+        """Return the attribution."""
+        return self._attr_attribution
+
+    @property
+    def entity_category(self) -> str | None:
+        """Return the category of the entity, if any."""
+        if hasattr(self, "_attr_entity_category"):
+            return self._attr_entity_category
+        if hasattr(self, "entity_description"):
+            return self.entity_description.entity_category
+        return None
+
     # DO NOT OVERWRITE
     # These properties and methods are either managed by Home Assistant or they
     # are used to perform a very specific function. Overwriting these may
@@ -337,8 +489,7 @@ class Entity(ABC):
         """Convert state to string."""
         if not self.available:
             return STATE_UNAVAILABLE
-        state = self.state
-        if state is None:
+        if (state := self.state) is None:
             return STATE_UNKNOWN
         if isinstance(state, float):
             # If the entity's state is a float, limit precision according to machine
@@ -381,53 +532,38 @@ class Entity(ABC):
 
         entry = self.registry_entry
         # pylint: disable=consider-using-ternary
-        name = (entry and entry.name) or self.name
-        if name is not None:
+        if (name := (entry and entry.name) or self.name) is not None:
             attr[ATTR_FRIENDLY_NAME] = name
 
-        icon = (entry and entry.icon) or self.icon
-        if icon is not None:
+        if (icon := (entry and entry.icon) or self.icon) is not None:
             attr[ATTR_ICON] = icon
 
-        entity_picture = self.entity_picture
-        if entity_picture is not None:
+        if (entity_picture := self.entity_picture) is not None:
             attr[ATTR_ENTITY_PICTURE] = entity_picture
 
-        assumed_state = self.assumed_state
-        if assumed_state:
+        if assumed_state := self.assumed_state:
             attr[ATTR_ASSUMED_STATE] = assumed_state
 
-        supported_features = self.supported_features
-        if supported_features is not None:
+        if (supported_features := self.supported_features) is not None:
             attr[ATTR_SUPPORTED_FEATURES] = supported_features
 
-        device_class = self.device_class
-        if device_class is not None:
+        if (device_class := self.device_class) is not None:
             attr[ATTR_DEVICE_CLASS] = str(device_class)
 
+        if (attribution := self.attribution) is not None:
+            attr[ATTR_ATTRIBUTION] = attribution
+
         end = timer()
 
         if end - start > 0.4 and not self._slow_reported:
             self._slow_reported = True
-            extra = ""
-            if "custom_components" in type(self).__module__:
-                extra = "Please report it to the custom component author."
-            else:
-                extra = (
-                    "Please create a bug report at "
-                    "https://github.com/home-assistant/core/issues?q=is%3Aopen+is%3Aissue"
-                )
-                if self.platform:
-                    extra += (
-                        f"+label%3A%22integration%3A+{self.platform.platform_name}%22"
-                    )
-
+            report_issue = self._suggest_report_issue()
             _LOGGER.warning(
-                "Updating state for %s (%s) took %.3f seconds. %s",
+                "Updating state for %s (%s) took %.3f seconds. Please %s",
                 self.entity_id,
                 type(self),
                 end - start,
-                extra,
+                report_issue,
             )
 
         # Overwrite properties that have been set in the config file.
@@ -504,7 +640,6 @@ class Entity(ABC):
             await self.parallel_updates.acquire()
 
         try:
-            # pylint: disable=no-member
             if hasattr(self, "async_update"):
                 task = self.hass.async_create_task(self.async_update())  # type: ignore
             elif hasattr(self, "update"):
@@ -519,8 +654,7 @@ class Entity(ABC):
             finished, _ = await asyncio.wait([task], timeout=SLOW_UPDATE_WARNING)
 
             for done in finished:
-                exc = done.exception()
-                if exc:
+                if exc := done.exception():
                     raise exc
                 return
 
@@ -633,7 +767,10 @@ class Entity(ABC):
         Not to be extended by integrations.
         """
         if self.platform:
-            info = {"domain": self.platform.platform_name}
+            info = {
+                "domain": self.platform.platform_name,
+                "custom_component": "custom_components" in type(self).__module__,
+            }
 
             if self.platform.config_entry:
                 info["source"] = SOURCE_CONFIG_ENTRY
@@ -674,7 +811,7 @@ class Entity(ABC):
         if data["action"] != "update":
             return
 
-        ent_reg = await self.hass.helpers.entity_registry.async_get_registry()
+        ent_reg = er.async_get(self.hass)
         old = self.registry_entry
         self.registry_entry = ent_reg.async_get(data["entity_id"])
         assert self.registry_entry is not None
@@ -728,19 +865,46 @@ class Entity(ABC):
             if self.parallel_updates:
                 self.parallel_updates.release()
 
+    def _suggest_report_issue(self) -> str:
+        """Suggest to report an issue."""
+        report_issue = ""
+        if "custom_components" in type(self).__module__:
+            report_issue = "report it to the custom component author."
+        else:
+            report_issue = (
+                "create a bug report at "
+                "https://github.com/home-assistant/core/issues?q=is%3Aopen+is%3Aissue"
+            )
+            if self.platform:
+                report_issue += (
+                    f"+label%3A%22integration%3A+{self.platform.platform_name}%22"
+                )
+
+        return report_issue
+
+
+@dataclass
+class ToggleEntityDescription(EntityDescription):
+    """A class that describes toggle entities."""
+
 
 class ToggleEntity(Entity):
     """An abstract class for entities that can be turned on and off."""
 
+    entity_description: ToggleEntityDescription
+    _attr_is_on: bool
+    _attr_state: None = None
+
     @property
-    def state(self) -> str:
+    @final
+    def state(self) -> str | None:
         """Return the state."""
         return STATE_ON if self.is_on else STATE_OFF
 
     @property
     def is_on(self) -> bool:
         """Return True if entity is on."""
-        raise NotImplementedError()
+        return self._attr_is_on
 
     def turn_on(self, **kwargs: Any) -> None:
         """Turn the entity on."""
